@@ -4,8 +4,13 @@ import requests
 from requests import HTTPError
 
 from freshdesk.v2.errors import (
-    FreshdeskAccessDenied, FreshdeskBadRequest, FreshdeskError, FreshdeskNotFound, FreshdeskRateLimited,
-    FreshdeskServerError, FreshdeskUnauthorized,
+    FreshdeskAccessDenied,
+    FreshdeskBadRequest,
+    FreshdeskError,
+    FreshdeskNotFound,
+    FreshdeskRateLimited,
+    FreshdeskServerError,
+    FreshdeskUnauthorized,
 )
 from freshdesk.v2.models import Agent, Comment, Company, Contact, Customer, Group, Role, Ticket, TicketField, TimeEntry
 
@@ -16,7 +21,7 @@ class TicketAPI(object):
 
     def get_ticket(self, ticket_id):
         """Fetches the ticket for the given ticket ID"""
-        url = 'tickets/%d' % ticket_id
+        url = "tickets/%d" % ticket_id
         ticket = self._api._get(url)
         return Ticket(**ticket)
 
@@ -28,16 +33,16 @@ class TicketAPI(object):
             ex: attachments = ('/path/to/attachment1', '/path/to/attachment2')
         """
 
-        url = 'tickets'
-        status = kwargs.get('status', 2)
-        priority = kwargs.get('priority', 1)
+        url = "tickets"
+        status = kwargs.get("status", 2)
+        priority = kwargs.get("priority", 1)
         data = {
-            'subject': subject,
-            'status': status,
-            'priority': priority,
+            "subject": subject,
+            "status": status,
+            "priority": priority,
         }
         data.update(kwargs)
-        if 'attachments' in data:
+        if "attachments" in data:
             ticket = self._create_ticket_with_attachment(url, data)
             return Ticket(**ticket)
 
@@ -45,40 +50,40 @@ class TicketAPI(object):
         return Ticket(**ticket)
 
     def _create_ticket_with_attachment(self, url, data):
-        attachments = data['attachments']
-        del data['attachments']
+        attachments = data["attachments"]
+        del data["attachments"]
         multipart_data = []
 
         for attachment in attachments:
             file_name = attachment.split("/")[-1:][0]
-            multipart_data.append(('attachments[]', (file_name, open(attachment, 'rb'), None)))
+            multipart_data.append(("attachments[]", (file_name, open(attachment, "rb"), None)))
 
         for key, value in data.copy().items():
             # Reformat ticket properties to work with the multipart/form-data encoding.
-            if isinstance(value, list) and not key.endswith('[]'):
-                data[key + '[]'] = value
+            if isinstance(value, list) and not key.endswith("[]"):
+                data[key + "[]"] = value
                 del data[key]
 
-        if 'custom_fields' in data and isinstance(data['custom_fields'], dict):
+        if "custom_fields" in data and isinstance(data["custom_fields"], dict):
             # Reformat custom fields to work with the multipart/form-data encoding.
-            for field, value in data['custom_fields'].items():
-                data['custom_fields[{}]'.format(field)] = value
-            del data['custom_fields']
+            for field, value in data["custom_fields"].items():
+                data["custom_fields[{}]".format(field)] = value
+            del data["custom_fields"]
 
         # Override the content type so that `requests` correctly sets it to multipart/form-data instead of JSON.
-        ticket = self._api._post(url, data=data, files=multipart_data, headers={'Content-Type': None})
+        ticket = self._api._post(url, data=data, files=multipart_data, headers={"Content-Type": None})
         return ticket
 
     def create_outbound_email(self, subject, description, email, email_config_id, **kwargs):
         """Creates an outbound email"""
-        url = 'tickets/outbound_email'
-        priority = kwargs.get('priority', 1)
+        url = "tickets/outbound_email"
+        priority = kwargs.get("priority", 1)
         data = {
-            'subject': subject,
-            'description': description,
-            'priority': priority,
-            'email': email,
-            'email_config_id': email_config_id,
+            "subject": subject,
+            "description": description,
+            "priority": priority,
+            "email": email,
+            "email_config_id": email_config_id,
         }
         data.update(kwargs)
         ticket = self._api._post(url, data=json.dumps(data))
@@ -86,13 +91,13 @@ class TicketAPI(object):
 
     def update_ticket(self, ticket_id, **kwargs):
         """Updates a ticket from a given ticket ID"""
-        url = 'tickets/%d' % ticket_id
+        url = "tickets/%d" % ticket_id
         ticket = self._api._put(url, data=json.dumps(kwargs))
         return Ticket(**ticket)
 
     def delete_ticket(self, ticket_id):
         """Delete the ticket for the given ticket ID"""
-        url = 'tickets/%d' % ticket_id
+        url = "tickets/%d" % ticket_id
         self._api._delete(url)
 
     def list_tickets(self, **kwargs):
@@ -109,27 +114,26 @@ class TicketAPI(object):
         Multiple filters are AND'd together.
         """
 
-        filter_name = 'new_and_my_open'
-        if 'filter_name' in kwargs:
-            filter_name = kwargs['filter_name']
-            del kwargs['filter_name']
+        filter_name = "new_and_my_open"
+        if "filter_name" in kwargs:
+            filter_name = kwargs["filter_name"]
+            del kwargs["filter_name"]
 
-        url = 'tickets'
+        url = "tickets"
         if filter_name is not None:
-            url += '?filter=%s&' % filter_name
+            url += "?filter=%s&" % filter_name
         else:
-            url += '?'
-        page = 1 if not 'page' in kwargs else kwargs['page']
-        per_page = 100 if not 'per_page' in kwargs else kwargs['per_page']
+            url += "?"
+        page = 1 if not "page" in kwargs else kwargs["page"]
+        per_page = 100 if not "per_page" in kwargs else kwargs["per_page"]
         tickets = []
 
         # Skip pagination by looping over each page and adding tickets if 'page' key is not in kwargs.
         # else return the requested page and break the loop
         while True:
-            this_page = self._api._get(url + 'page=%d&per_page=%d'
-                                       % (page, per_page), kwargs)
+            this_page = self._api._get(url + "page=%d&per_page=%d" % (page, per_page), kwargs)
             tickets += this_page
-            if len(this_page) < per_page or 'page' in kwargs:
+            if len(this_page) < per_page or "page" in kwargs:
                 break
             page += 1
 
@@ -137,15 +141,15 @@ class TicketAPI(object):
 
     def list_new_and_my_open_tickets(self):
         """List all new and open tickets."""
-        return self.list_tickets(filter_name='new_and_my_open')
+        return self.list_tickets(filter_name="new_and_my_open")
 
     def list_watched_tickets(self):
         """List watched tickets, closed or open."""
-        return self.list_tickets(filter_name='watching')
+        return self.list_tickets(filter_name="watching")
 
     def list_deleted_tickets(self):
         """Lists all deleted tickets."""
-        return self.list_tickets(filter_name='deleted')
+        return self.list_tickets(filter_name="deleted")
 
     def filter_tickets(self, query, **kwargs):
         """Filter tickets by a given query string. The query string must be in
@@ -154,20 +158,19 @@ class TicketAPI(object):
 
         query = "(ticket_field:integer OR ticket_field:'string') AND ticket_field:boolean"
         """
-        if(len(query) > 512):
-            raise AttributeError('Query string can have up to 512 characters')
-        
-        url = 'search/tickets?'
-        page = 1 if not 'page' in kwargs else kwargs['page']
+        if len(query) > 512:
+            raise AttributeError("Query string can have up to 512 characters")
+
+        url = "search/tickets?"
+        page = 1 if not "page" in kwargs else kwargs["page"]
         per_page = 30
 
         tickets = []
         while True:
-            this_page = self._api._get(url + 'page={}&query="{}"'.format(page, query),
-                                        kwargs)
-            this_page = this_page['results']
+            this_page = self._api._get(url + 'page={}&query="{}"'.format(page, query), kwargs)
+            this_page = this_page["results"]
             tickets += this_page
-            if len(this_page) < per_page or page == 10 or 'page' in kwargs:
+            if len(this_page) < per_page or page == 10 or "page" in kwargs:
                 break
             page += 1
 
@@ -179,21 +182,21 @@ class CommentAPI(object):
         self._api = api
 
     def list_comments(self, ticket_id):
-        url = 'tickets/%d/conversations' % ticket_id
+        url = "tickets/%d/conversations" % ticket_id
         comments = []
         for c in self._api._get(url):
             comments.append(Comment(**c))
         return comments
 
     def create_note(self, ticket_id, body, **kwargs):
-        url = 'tickets/%d/notes' % ticket_id
-        data = {'body': body}
+        url = "tickets/%d/notes" % ticket_id
+        data = {"body": body}
         data.update(kwargs)
         return Comment(**self._api._post(url, data=json.dumps(data)))
 
     def create_reply(self, ticket_id, body, **kwargs):
-        url = 'tickets/%d/reply' % ticket_id
-        data = {'body': body}
+        url = "tickets/%d/reply" % ticket_id
+        data = {"body": body}
         data.update(kwargs)
         return Comment(**self._api._post(url, data=json.dumps(data)))
 
@@ -203,23 +206,22 @@ class GroupAPI(object):
         self._api = api
 
     def list_groups(self, **kwargs):
-        url = 'groups?'
-        page = 1 if not 'page' in kwargs else kwargs['page']
-        per_page = 100 if not 'per_page' in kwargs else kwargs['per_page']
+        url = "groups?"
+        page = 1 if not "page" in kwargs else kwargs["page"]
+        per_page = 100 if not "per_page" in kwargs else kwargs["per_page"]
 
         groups = []
         while True:
-            this_page = self._api._get(url + 'page=%d&per_page=%d'
-                                       % (page, per_page), kwargs)
+            this_page = self._api._get(url + "page=%d&per_page=%d" % (page, per_page), kwargs)
             groups += this_page
-            if len(this_page) < per_page or 'page' in kwargs:
+            if len(this_page) < per_page or "page" in kwargs:
                 break
             page += 1
 
         return [Group(**g) for g in groups]
 
     def get_group(self, group_id):
-        url = 'groups/%s' % group_id
+        url = "groups/%s" % group_id
         return Group(**self._api._get(url))
 
 
@@ -250,19 +252,18 @@ class ContactAPI(object):
 
         """
 
-        url = 'contacts?'
-        page = 1 if not 'page' in kwargs else kwargs['page']
-        per_page = 100 if not 'per_page' in kwargs else kwargs['per_page']
+        url = "contacts?"
+        page = 1 if not "page" in kwargs else kwargs["page"]
+        per_page = 100 if not "per_page" in kwargs else kwargs["per_page"]
 
         contacts = []
 
         # Skip pagination by looping over each page and adding tickets if 'page' key is not in kwargs.
         # else return the requested page and break the loop
         while True:
-            this_page = self._api._get(url + 'page=%d&per_page=%d'
-                                       % (page, per_page), kwargs)
+            this_page = self._api._get(url + "page=%d&per_page=%d" % (page, per_page), kwargs)
             contacts += this_page
-            if len(this_page) < per_page or 'page' in kwargs:
+            if len(this_page) < per_page or "page" in kwargs:
                 break
 
             page += 1
@@ -271,43 +272,40 @@ class ContactAPI(object):
 
     def create_contact(self, *args, **kwargs):
         """Creates a contact"""
-        url = 'contacts'
-        data = {
-            'view_all_tickets': False,
-            'description': 'Freshdesk Contact'
-        }
+        url = "contacts"
+        data = {"view_all_tickets": False, "description": "Freshdesk Contact"}
         data.update(kwargs)
         return Contact(**self._api._post(url, data=json.dumps(data)))
 
     def get_contact(self, contact_id):
-        url = 'contacts/%d' % contact_id
+        url = "contacts/%d" % contact_id
         return Contact(**self._api._get(url))
 
     def update_contact(self, contact_id, **data):
-        url = 'contacts/%d' % contact_id
+        url = "contacts/%d" % contact_id
         return Contact(**self._api._put(url, data=json.dumps(data)))
 
     def soft_delete_contact(self, contact_id):
-        url = 'contacts/%d' % contact_id
+        url = "contacts/%d" % contact_id
         self._api._delete(url)
 
     def restore_contact(self, contact_id):
-        url = 'contacts/%d/restore' % contact_id
+        url = "contacts/%d/restore" % contact_id
         self._api._put(url)
 
     def permanently_delete_contact(self, contact_id, force=True):
-        url = 'contacts/%d/hard_delete?force=%r' % (contact_id, force)
+        url = "contacts/%d/hard_delete?force=%r" % (contact_id, force)
         self._api._delete(url)
 
     def make_agent(self, contact_id, **kwargs):
-        url = 'contacts/%d/make_agent' % contact_id
+        url = "contacts/%d/make_agent" % contact_id
         data = {
-            'occasional': False,
-            'ticket_scope': 2,
+            "occasional": False,
+            "ticket_scope": 2,
         }
         data.update(kwargs)
         contact = self._api._put(url, data=json.dumps(data))
-        return self._api.agents.get_agent(contact['agent']['id'])
+        return self._api.agents.get_agent(contact["agent"]["id"])
 
 
 class CustomerAPI(object):
@@ -315,7 +313,7 @@ class CustomerAPI(object):
         self._api = api
 
     def get_customer(self, company_id):
-        url = 'customers/%s' % company_id
+        url = "customers/%s" % company_id
         return Customer(**self._api._get(url))
 
     def get_customer_from_contact(self, contact):
@@ -327,23 +325,22 @@ class CompanyAPI(object):
         self._api = api
 
     def get_company(self, company_id):
-        url = 'companies/%s' % company_id
+        url = "companies/%s" % company_id
         return Company(**self._api._get(url))
 
     def list_companies(self, **kwargs):
-        url = 'companies?'
-        page = 1 if not 'page' in kwargs else kwargs['page']
-        per_page = 100 if not 'per_page' in kwargs else kwargs['per_page']
+        url = "companies?"
+        page = 1 if not "page" in kwargs else kwargs["page"]
+        per_page = 100 if not "per_page" in kwargs else kwargs["per_page"]
 
         companies = []
 
         # Skip pagination by looping over each page and adding tickets if 'page' key is not in kwargs.
         # else return the requested page and break the loop
         while True:
-            this_page = self._api._get(url + 'page=%d&per_page=%d'
-                                       % (page, per_page), kwargs)
+            this_page = self._api._get(url + "page=%d&per_page=%d" % (page, per_page), kwargs)
             companies += this_page
-            if len(this_page) < per_page or 'page' in kwargs:
+            if len(this_page) < per_page or "page" in kwargs:
                 break
 
             page += 1
@@ -356,14 +353,14 @@ class RoleAPI(object):
         self._api = api
 
     def list_roles(self):
-        url = 'roles'
+        url = "roles"
         roles = []
         for r in self._api._get(url):
             roles.append(Role(**r))
         return roles
 
     def get_role(self, role_id):
-        url = 'roles/%s' % role_id
+        url = "roles/%s" % role_id
         return Role(**self._api._get(url))
 
 
@@ -372,16 +369,16 @@ class TimeEntryAPI(object):
         self._api = api
 
     def list_time_entries(self, ticket_id=None):
-        url = 'tickets/time_entries'
+        url = "tickets/time_entries"
         if ticket_id is not None:
-            url = 'tickets/%d/time_entries' % ticket_id
+            url = "tickets/%d/time_entries" % ticket_id
         timeEntries = []
         for r in self._api._get(url):
             timeEntries.append(TimeEntry(**r))
         return timeEntries
 
     def get_role(self, role_id):
-        url = 'roles/%s' % role_id
+        url = "roles/%s" % role_id
         return Role(**self._api._get(url))
 
 
@@ -390,11 +387,11 @@ class TicketFieldAPI(object):
         self._api = api
 
     def list_ticket_fields(self, **kwargs):
-        url = 'ticket_fields'
+        url = "ticket_fields"
         ticket_fields = []
 
-        if 'type' in kwargs:
-            url = "{}?type={}".format(url, kwargs['type'])
+        if "type" in kwargs:
+            url = "{}?type={}".format(url, kwargs["type"])
 
         for tf in self._api._get(url):
             ticket_fields.append(TicketField(**tf))
@@ -422,19 +419,18 @@ class AgentAPI(object):
         Multiple filters are AND'd together.
         """
 
-        url = 'agents?'
-        page = 1 if not 'page' in kwargs else kwargs['page']
-        per_page = 100 if not 'per_page' in kwargs else kwargs['per_page']
+        url = "agents?"
+        page = 1 if not "page" in kwargs else kwargs["page"]
+        per_page = 100 if not "per_page" in kwargs else kwargs["per_page"]
 
         agents = []
 
         # Skip pagination by looping over each page and adding tickets if 'page' key is not in kwargs.
         # else return the requested page and break the loop
         while True:
-            this_page = self._api._get(url + 'page=%d&per_page=%d'
-                                       % (page, per_page), kwargs)
+            this_page = self._api._get(url + "page=%d&per_page=%d" % (page, per_page), kwargs)
             agents += this_page
-            if len(this_page) < per_page or 'page' in kwargs:
+            if len(this_page) < per_page or "page" in kwargs:
                 break
             page += 1
 
@@ -442,23 +438,23 @@ class AgentAPI(object):
 
     def get_agent(self, agent_id):
         """Fetches the agent for the given agent ID"""
-        url = 'agents/%s' % agent_id
+        url = "agents/%s" % agent_id
         return Agent(**self._api._get(url))
 
     def update_agent(self, agent_id, **kwargs):
         """Updates an agent"""
-        url = 'agents/%s' % agent_id
+        url = "agents/%s" % agent_id
         agent = self._api._put(url, data=json.dumps(kwargs))
         return Agent(**agent)
 
     def delete_agent(self, agent_id):
         """Delete the agent for the given agent ID"""
-        url = 'agents/%d' % agent_id
+        url = "agents/%d" % agent_id
         self._api._delete(url)
 
     def currently_authenticated_agent(self):
         """Fetches currently logged in agent"""
-        url = 'agents/me'
+        url = "agents/me"
         return Agent(**self._api._get(url))
 
 
@@ -474,12 +470,12 @@ class API(object):
           .tickets:  the Ticket API
         """
 
-        self._api_prefix = 'https://{}/api/v2/'.format(domain.rstrip('/'))
+        self._api_prefix = "https://{}/api/v2/".format(domain.rstrip("/"))
         self._session = requests.Session()
-        self._session.auth = (api_key, 'unused_with_api_key')
+        self._session.auth = (api_key, "unused_with_api_key")
         self._session.verify = verify
         self._session.proxies = proxies
-        self._session.headers = {'Content-Type': 'application/json'}
+        self._session.headers = {"Content-Type": "application/json"}
 
         self.tickets = TicketAPI(self)
         self.comments = CommentAPI(self)
@@ -491,9 +487,8 @@ class API(object):
         self.roles = RoleAPI(self)
         self.ticket_fields = TicketFieldAPI(self)
 
-        if domain.find('freshdesk.com') < 0:
-            raise AttributeError('Freshdesk v2 API works only via Freshdesk'
-                                 'domains and not via custom CNAMEs')
+        if domain.find("freshdesk.com") < 0:
+            raise AttributeError("Freshdesk v2 API works only via Freshdesk" "domains and not via custom CNAMEs")
         self.domain = domain
 
     def _action(self, req):
@@ -502,12 +497,12 @@ class API(object):
         except ValueError:
             j = {}
 
-        error_message = 'Freshdesk Request Failed'
-        if 'errors' in j:
-            error_message = '{}: {}'.format(j.get('description'), j.get('errors'))
-        elif 'message' in j:
-            error_message = j['message']
-            
+        error_message = "Freshdesk Request Failed"
+        if "errors" in j:
+            error_message = "{}: {}".format(j.get("description"), j.get("errors"))
+        elif "message" in j:
+            error_message = j["message"]
+
         if req.status_code == 400:
             raise FreshdeskBadRequest(error_message)
         elif req.status_code == 401:
@@ -518,10 +513,11 @@ class API(object):
             raise FreshdeskNotFound(error_message)
         elif req.status_code == 429:
             raise FreshdeskRateLimited(
-                '429 Rate Limit Exceeded: API rate-limit has been reached until {} seconds. See '
-                'http://freshdesk.com/api#ratelimit'.format(req.headers.get('Retry-After')))
+                "429 Rate Limit Exceeded: API rate-limit has been reached until {} seconds. See "
+                "http://freshdesk.com/api#ratelimit".format(req.headers.get("Retry-After"))
+            )
         elif 500 < req.status_code < 600:
-            raise FreshdeskServerError('{}: Server Error'.format(req.status_code))
+            raise FreshdeskServerError("{}: Server Error".format(req.status_code))
 
         # Catch any other errors
         try:
